/**
 * @file frametimer.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Times frames
 * THIS CLASS IS NOT AIMED TO BE UNIVERSAL - IT IS SPECIALLY
 * CREATED FOR USE IN DMXPLAYER
**/
#include "frametimer.h"
#include <unistd.h>
#include <pthread.h>
#include <sys/time.h>

/* globals */
pthread_t thread;
int  running   = 0;
int  terminate;
long frame_nr;
long frameduration;
void (*callback)(long) = 0;
pthread_mutex_t rw_mutex;



/** 
 * Get a timestamp in microseconds
**/
long get_time_usecs()
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return (tv.tv_sec*1000*1000) + tv.tv_usec;
}



/**
 * Timer main thread
**/
void *frametimer_main(void *ptr)
{
	long start;
	/* run */
	while(!terminate)
	{
		start = get_time_usecs();
		frametimer_lock();
			callback(frame_nr);
			frame_nr++;
		frametimer_unlock();
		/* wait to give other threads a chance to lock */
		usleep(5);
		while( (get_time_usecs()-start) < frameduration)
			usleep(50);
	}
	/* terminate */
	terminate = 0;
	running = 0;
	return NULL;
}



/**
 * Start the frame timer
 * @return 1 on success, 0 on failure
**/
int frametimer_start(void (*newcallback)(long), int fps)
{
	int ret;
	/* already running? */
	if(running != 0) return 0;
	running = 1;
	/* don't terminate now */
	terminate = 0;
	/* first frame nr */
	frame_nr = 0;
	/* set frame duration  time=(1000*1000)/fps */
	frameduration = 1000000/fps;
	/* set callback function */
	callback = newcallback;
	/* set mutex */
	if( pthread_mutex_init(&rw_mutex, NULL) != 0 ) return 0;
	/* start main thread */
	ret = pthread_create(&thread, NULL, frametimer_main, (void*) NULL);
	/* success */
	return (ret == 0);
}



/**
 * Locks the frame timers mutex
**/
void frametimer_lock()
{
	pthread_mutex_lock(&rw_mutex);
}



/**
 * Unlocks the frame timers mutex
**/
void frametimer_unlock()
{
	pthread_mutex_unlock(&rw_mutex);
}



/**
 * Stops the frame timer
**/
void frametimer_stop()
{
	terminate = 1;
	pthread_join(thread, NULL);
	pthread_mutex_destroy(&rw_mutex);
}



/**
 * Get current frame number
**/
long frametimer_getframe()
{
	return frame_nr;
}



/**
 * Get current frame number
**/
void frametimer_sync()
{
	frame_nr = 0;
}
