/**
 * @file listing.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Jan-2013 / All rights reserved. 
 * ---------------------------------------------------------
 * Provides gui content
**/
#include "listing.h"
#include "dmxgui_tools.h"
#include "../dmxplayer.h"

int selected = 0;
int offset   = 0;
int global_selected = 0;

/* A key event in active play mode */
void listing_keyevent(int c)
{
	switch(c)
	{
		/* toogle channel */
		case KEY_UP:
			if(selected > 0) selected--;
			else
			{
				if(offset > 0) offset--;
			}
			break;
		case KEY_DOWN:
			selected++;
			if(selected >= (LINES-9)) { offset += 1; selected--; }
			if(offset   > DMXPLAYER_PROGRAMS-(LINES-9)) offset = DMXPLAYER_PROGRAMS-(LINES-9);
			if(selected >= DMXPLAYER_PROGRAMS) selected = DMXPLAYER_PROGRAMS - 1;
			break;
		/* toogle first program */
		case KEY_HOME:
			selected = 0;
			offset   = 0;
			break;
		/* toogle last program */
		case KEY_END:
			selected = (LINES-9)-1;
			offset   = DMXPLAYER_PROGRAMS-(LINES-9);
			break;
		/* toogle play */
		case 'P':
			dmxplayer_toogle_program(offset+selected);
			break;
		/* toogle channel */
		case KEY_ENTER:
		case '\r':
		case '\n':
			global_selected = offset+selected;
			break;
	}
}


/* Print active play window */
void listing_printer(WINDOW* wnd)
{
	int n, x, y, acnt, necnt;
	char key;
	dmxprogram_t *p;
	/* draw window frame */
	wbkgd(wnd, COLOR_PAIR(1));
	box(wnd, 0, 0);
	mvwprintw(wnd, 3, 2, "Chaser Listing:");

	/* draw window content */
	n = 0; acnt = 0; necnt = 0;
	while( (p = dmxplayer_getprogram(offset+n++)) && (n<(LINES-8)) )
	{
		y = 3+n;
		x = 2;
		dmxgui_clear_line(wnd, y);
		/* print arrow */
		if( (n-1) == selected ) mvwprintw(wnd, y, x, "->");
		/* set attributes */
		if( (offset+n-1) == global_selected ) { wattron(wnd, COLOR_PAIR(3)); wattron(wnd, A_REVERSE); } 
		else wattron(wnd, COLOR_PAIR(2));
		if(p->size > 0) necnt++;
		/* check if active */
		if( (offset+n-1) < 26 ) key = 'A'+(offset+n-1);
		else key = '-';
		if(p->active) 
		{
			
			if(p->group == 0)
				mvwprintw(wnd, y, x+3,               "[%02d] [%c] [PLAY] %s", offset+n, key, p->name);
			else
				mvwprintw(wnd, y, x+3,               "[%02d] [%c] [PLAY] %s (%d)", offset+n, key, p->name, p->group);
			acnt++;
		}
		else
		{
			if(p->group == 0)
				mvwprintw(wnd, y, x+3,               "[%02d] [%c] [STOP] %s", offset+n, key, p->name);
			else
				mvwprintw(wnd, y, x+3,               "[%02d] [%c] [STOP] %s (%d)", offset+n, key, p->name, p->group);
		}
		/* remove attributes */
		if( (offset+n-1) == global_selected ) { wattroff(wnd, COLOR_PAIR(3)); wattroff(wnd, A_REVERSE); }
		else wattroff(wnd, COLOR_PAIR(2));
	}

	mvwprintw(wnd, 3, (COLS/2+4), "Statistic:");
	wattron(wnd, COLOR_PAIR(5));
	mvwprintw(wnd, 4, (COLS/2+4), "%02d active", acnt);
	mvwprintw(wnd, 5, (COLS/2+4), "%02d inactive", 99-acnt);
	mvwprintw(wnd, 6, (COLS/2+4), "%02d non-empty", necnt);
	mvwprintw(wnd, 7, (COLS/2+4), "%d%% free     ", ((99-necnt)*100/99));
	mvwprintw(wnd,  9, (COLS/2+4), "Press Enter to select");
	mvwprintw(wnd, 10, (COLS/2+4), "Press SHIFT+P to toogle play");
	wattroff(wnd, COLOR_PAIR(5));
	/* show information */	
	dmxgui_print_key_bindings(wnd);
}


/* get global selected value */
int listing_getSelectedProgram()
{
	return global_selected;
}


/* set global selected value */
void listing_setSelectedProgram(int v)
{
	global_selected = v;
}
