/**
 * @file modules/dmx4all.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Nov-2012 / All rights reserved. 
 * ---------------------------------------------------------
 * Implements DMX4ALL interface
**/
#include "dmx4all.h"
#include <lindmx.h>
#include <syslog.h>
#include <stdlib.h>

/* dmx context */
LINDMX dmx       = NULL;


/**
 * dmx4all_init()
 * Init DMX4ALL Micro USB Interface
 * @param maxchannel    The highest channel number created
 * @return 1 on success, 0 on failure
**/
int dmx4all_init(int maxchannel)
{
	/* create context */
	dmx = lindmx_create(maxchannel);
	/* Error while creating the interface object (e.g. no root rights, no interface found) */
	if(dmx == NULL) 
	{	
     	syslog(LOG_ERR, "CAN NOT CONNECT TO DMX4ALL INTERFACE - NO ROOT ACCESS?");
		return 0;
	}
	return 1;
}



/**
 * dmx4all_destroy()
 * Clean up interface
**/
void dmx4all_destroy()
{
	/* create context */
	lindmx_destroy(dmx);
	dmx = 0;
}



/**
 * dmx4all_reset_channels()
 * Resets all channels to zero
 * @param maxchannel    The highest channel number created
 * @return 1 on success, 0 on failure
**/
int dmx4all_reset_channels()
{
	int ret;
	if( dmx == NULL ) return 0;
	/* reset */
	ret = lindmx_reset_channels(dmx);
	/* check for error */
	if(ret <= 0) return 0;
	return 1;
}



/**
 * dmx4all_set_all()
 * Sets all channels to a given value
 * @param value    Value to set all channels to
 * @return 1 on success, 0 on failure
**/
int dmx4all_set_all(int value)
{
	int ret;
	if( dmx == NULL ) return 0;
	/* reset */
	ret = lindmx_set_all(value, dmx);
	/* check for error */
	if(ret <= 0) return 0;
	return 1;
}



/**
 * dmx4all_set()
 * Sets a channel to a given value
 * @param channel  Channel to set
 * @param value    Value to set to
 * @return 1 on success, 0 on failure
**/
int dmx4all_set(int channel, int value)
{
	int ret;
	if( dmx == NULL ) return 0;
	/* reset */
	ret = lindmx_set(channel, value, dmx);
	/* check for error */
	if(ret <= 0) return 0;
	return 1;
}
